export interface Cache {
  /**
   * @update 获取缓存
   */
  get(key: string): Promise<any>;
  /**
   * @update 设置缓存
   */
  set(key: string, value: any): Promise<any>;
  /**
   * @update 删除缓存
   */
  del(): Promise<void>;
}

export type CacheLoginUser = {
  user?: any;
  id?: string;
  accessToken?: string;
  refreshToken?: string;
};
type CacheLogin = {
  loginUsers: CacheLoginUser[];
} & CacheLoginUser;

export type CacheStore<T = any> = {
  name: string;
  cacheData: CacheLogin;
  /**
   * 实际操作的cache
   */
  cache: T;

  /**
   * 设置当前用户
   */
  setLoginUser(user: CacheLoginUser): Promise<void>;
  /**
   * 获取当前用户
   */
  getCurrentUser(): Promise<CacheLoginUser>;
  /**
   * 获取当前用户列表
   */
  getCurrentUserList(): Promise<CacheLoginUser[]>;
  /**
   * 获取缓存的refreshToken
   */
  getRefreshToken(): Promise<string>;
  /**
   * 获取缓存的accessToken
   */
  getAccessToken(): Promise<string>;
  /**
   * 初始化
   */
  init(): Promise<void>;
  /**
   * 清除当前用户
   */
  clearCurrentUser(): Promise<void>;
  /**
   * 清除所有用户
   */
  clearAll(): Promise<void>;
} & Cache;

type LoginCacheStoreOpts = {
  name: string;
  cache: Cache;
};
export class LoginCacheStore implements CacheStore<any> {
  cache: Cache;
  name: string;
  cacheData: CacheLogin;
  constructor(opts: LoginCacheStoreOpts) {
    if (!opts.cache) {
      throw new Error('cache is required');
    }
    // @ts-ignore
    this.cache = opts.cache;
    this.cacheData = {
      loginUsers: [],
      user: undefined,
      id: undefined,
      accessToken: undefined,
      refreshToken: undefined,
    };
    this.name = opts.name;
  }
  /**
   * 设置缓存
   * @param key
   * @param value
   * @returns
   */
  async set(key: string, value: CacheLogin) {
    await this.cache.set(key, value);
    return value;
  }
  /**
   * 删除缓存
   */
  async del() {
    await this.cache.del();
  }
  get(key: string): Promise<CacheLogin> {
    return this.cache.get(key);
  }

  async init() {
    this.cacheData = (await this.get(this.name)) || {
      loginUsers: [],
      user: null,
      id: null,
      accessToken: null,
      refreshToken: null,
    };
  }
  /**
   * 设置当前用户
   * @param user
   */
  async setLoginUser(user: CacheLoginUser) {
    const has = this.cacheData.loginUsers.find((u) => u.id === user.id);
    if (has) {
      this.cacheData.loginUsers = this.cacheData?.loginUsers?.filter((u) => u?.id && u.id !== user.id);
    }
    this.cacheData.loginUsers.push(user);
    this.cacheData.user = user.user;
    this.cacheData.id = user.id;
    this.cacheData.accessToken = user.accessToken;
    this.cacheData.refreshToken = user.refreshToken;
    await this.set(this.name, this.cacheData);
  }

  getCurrentUser(): Promise<CacheLoginUser> {
    const cacheData = this.cacheData;
    return Promise.resolve(cacheData.user);
  }
  getCurrentUserList(): Promise<CacheLoginUser[]> {
    return Promise.resolve(this.cacheData.loginUsers.filter((u) => u?.id));
  }
  getRefreshToken(): Promise<string> {
    const cacheData = this.cacheData;
    return Promise.resolve(cacheData.refreshToken || '');
  }
  getAccessToken(): Promise<string> {
    const cacheData = this.cacheData;
    return Promise.resolve(cacheData.accessToken || '');
  }

  async clearCurrentUser() {
    const user = await this.getCurrentUser();
    const has = this.cacheData.loginUsers.find((u) => u.id === user.id);
    if (has) {
      this.cacheData.loginUsers = this.cacheData?.loginUsers?.filter((u) => u?.id && u.id !== user.id);
    }
    this.cacheData.user = undefined;
    this.cacheData.id = undefined;
    this.cacheData.accessToken = undefined;
    this.cacheData.refreshToken = undefined;
    await this.set(this.name, this.cacheData);
  }
  async clearAll() {
    this.cacheData.loginUsers = [];
    this.cacheData.user = undefined;
    this.cacheData.id = undefined;
    this.cacheData.accessToken = undefined;
    this.cacheData.refreshToken = undefined;
    await this.set(this.name, this.cacheData);
  }
}
