import { Cache } from './login-cache.ts';
import { homedir } from 'node:os';
import { join, dirname } from 'node:path';
import { readFileSync } from 'node:fs';
import { readFile, writeFile, unlink, stat, mkdir } from 'node:fs/promises';
export const fileExists = async (filePath: string, createIfNotExists = false) => {
  try {
    await stat(filePath);
    return true;
  } catch (error) {
    if (createIfNotExists) {
      await mkdir(dirname(filePath), { recursive: true });
      return false;
    }
    return false;
  }
};
export const readConfigFile = (filePath: string) => {
  try {
    const data = readFileSync(filePath, 'utf-8');
    const jsonData = JSON.parse(data);
    return jsonData;
  } catch (error) {
    return {};
  }
};
export const getHostName = () => {
  const configDir = join(homedir(), '.config', 'envision');
  const configFile = join(configDir, 'config.json');
  const config = readConfigFile(configFile);
  const baseURL = config.baseURL || 'https://kevisual.cn';
  const hostname = new URL(baseURL).hostname;
  return hostname;
};
export class StorageNode implements Storage {
  cacheData: any;
  filePath: string;
  constructor() {
    this.cacheData = {};
    const configDir = join(homedir(), '.config', 'envision');
    const hostname = getHostName();
    this.filePath = join(configDir, 'config', `${hostname}-storage.json`);
    fileExists(this.filePath, true);
  }
  async loadCache() {
    const filePath = this.filePath;
    try {
      const data = await readFile(filePath, 'utf-8');
      const jsonData = JSON.parse(data);
      this.cacheData = jsonData;
    } catch (error) {
      this.cacheData = {};
      await writeFile(filePath, JSON.stringify(this.cacheData, null, 2));
    }
  }
  get length() {
    return Object.keys(this.cacheData).length;
  }
  getItem(key: string) {
    return this.cacheData[key];
  }
  setItem(key: string, value: any) {
    this.cacheData[key] = value;
    writeFile(this.filePath, JSON.stringify(this.cacheData, null, 2));
  }
  removeItem(key: string) {
    delete this.cacheData[key];
    writeFile(this.filePath, JSON.stringify(this.cacheData, null, 2));
  }
  clear() {
    this.cacheData = {};
    writeFile(this.filePath, JSON.stringify(this.cacheData, null, 2));
  }
  key(index: number) {
    return Object.keys(this.cacheData)[index];
  }
}
export class LoginNodeCache implements Cache {
  name: string;
  cacheData: any;
  filePath: string;
  constructor(name: string) {
    this.name = name;
    const hostname = getHostName();
    this.filePath = join(homedir(), '.config', 'envision', 'config', `${hostname}-${name}.json`);
    fileExists(this.filePath, true);
    this.loadCache(this.filePath);
  }

  async loadCache(filePath: string) {
    try {
      const data = await readFile(filePath, 'utf-8');
      const jsonData = JSON.parse(data);
      this.cacheData = jsonData;
    } catch (error) {
      const defaultData = { loginUsers: [] };
      this.cacheData = defaultData;
      await writeFile(filePath, JSON.stringify(defaultData, null, 2));
    }
  }

  async get() {
    return this.cacheData;
  }
  async set(key: string, value: any) {
    this.cacheData = value;
    await writeFile(this.filePath, JSON.stringify(this.cacheData, null, 2));
  }
  async del() {
    await unlink(this.filePath);
  }
}
